



python -c"import bcolz; bcolz.test(heavy=True)"
IF %ERRORLEVEL% NEQ 0 exit /B 1
python -c"import bcolz; assert set(bcolz.blosc_compressor_list()) == set(['blosclz', 'lz4', 'lz4hc', 'snappy', 'zlib', 'zstd'])"
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
